/**
 * @file Implémentation des requêtes GraphQL.
 * @author akka vodol
*/
import { request } from 'https';
import _ from 'lodash';
import { assertBinaryExpression } from 'babel-types';
import knex from '../../db/knex_router';

import * as connectors from './connectors/connectors.js';
import { connect } from 'http2';

/** 
 * @description Résolveurs des différentes requêtes GraphQL
*/
export const resolvers = {
    // @rights user
    Query: {

        asAdmin: async function (obj, args, context){
            if(await connectors.hasAdminRights(context.user, args.groupUID))
                return {groupUID : args.groupUID};
            else
                throw "You do not have admin rights over this group";
        },

        asMember: async function (obj, args, context){
            let res = await connectors.isMember(context.user, args.groupUID);
            console.log(res);
            if(res/*await connectors.isMember(context.user, args.groupUID) || connectors.hasAdminRights(context.user, args.groupUID)*/){
                return {groupUID : args.groupUID};
            }else
                throw "You do not have member rights over this group";
        },

        accessGroups: (obj, args, context) => {
            return {};
        },

        accessPosts: (obj, args, context) => {
            return {};
        },

        accessUsers: (obj, args, context) => {
            return {};
        },
    },

    // @rights user
    GroupQuery : {
        allGroups: (obj, args, context) => {
            return connectors.getAllVisibleGroups(context.user);
        },
        allSimpleGroups: (obj, args, context) => {
            return connectors.getAllVisibleSimpleGroups(context.user);
        },

        group: (obj, args, context) => {
            return connectors.getGroupIfVisible(context.user, args.uid);
        },
        simpleGroup: (obj, args, context) => {
            return connectors.getSimpleGroupIfVisible(context.user, args.uid);
        },
        metaGroup: (obj, args, context) => {
            return connectors.getMetaGroupIfVisible(context.user, args.uid);
        }
    },

    // @rights use
    MessageQuery : {
        allPosts: (obj, args, context) => {
            return knex.select().from('posts');
        },
        message: (obj, args, context) => {
            const result = knex.select().from('messages').where('id','=',args.id);
            return result.then((res) => res[0]);
        },
    },

    // @rights user
    UserQuery: {
        user: (obj, args, context) => {
            
            return connectors.getUser(context.user,args.uid);
        },

        searchTOL: (obj, args, context) => {
            console.log(args);
            return connectors.repliquerTOLdesIds(context.user, {
                givenName: args.givenName,
                lastName: args.lastName,
                nickname: args.nickname,
                nationality: args.nationality,
                school: args.school,
                promotion: args.promotion,
                groups: args.groups,
                studies: args.studies,
                sport: args.sport,
                phone: args.phone,
                mail: args.mail,
                adress: args.adress,
                ip: args.ip
            });
        }
    },

    // @rights admin(obj.groupUID)
    AdminQuery: {
        isAdmin: (obj, args, context) => {
            return true;
        },

        allRequests: async function(obj, args, context){
            let ujg_requests = await connectors.getUserJoinGroupRequests(context.user, obj.groupUID);
            let gje_requests = await connectors.getGroupJoinEventRequests(context.user, obj.groupUID);
            let yghe_requests = await connectors.getYourGroupHostEventRequests(context.user, obj.groupUID);
            return ujg_requests.concat(gje_requests).concat(yghe_requests);
        }
    },

    // @rights member(obj.groupUID)
    MemberQuery: {
        isMember: (obj, args, context) => {
            return true;
        },

        allMembers: async function (obj, args, context){
            let cb = await connectors.getMetaGroupsFromCallbacks(context.user, obj.groupUID);
            return cb(knex);
        }
    },

    SpeakerQuery: {

        allRequests: async function(obj, args, context){
            let gje_requests = await connectors.getGroupJoinEventRequests(context.user, obj.groupUID);
            let yghe_requests = await connectors.getYourGroupHostEventRequests(context.user, obj.groupUID);
            return gje_requests.concat(yghe_requests);
        }
    },

    Request : {
        __resolveType : function(obj){
            return obj.type;
        }
    },

    // @rights admin(obj.groupUID)
    UserJoinGroup: {
        user : (obj, args, context) => {
            return connectors.getUser(context.user, obj.useruid);
            /*return connectors.getUser(context.user, "quentin.gendre");
            if(obj.useruid === "anatole.romon"){
                return connectors.getUser(context.user, "anatole.romon").then(res => {
                    return connectors.getUser(context.user, "quentin.gendre");
                });
            }else{
                return new Promise( (resolve, reject) => {
                    resolve({givenName : "patrick"});
                });
            }*/
        }
    },

    // @rights speaker(obj.groupUID)
    GroupJoinEvent : {
        event: (obj, args, context) => {
            return connectors.getEvent(context.user, obj.eventuid);
        },
        groupWantingToJoin: (obj, args, context) => {
            return connectors.getGroup(context.user, obj.senderuid);
        }
    },

    // @rights speaker(obj.groupUID)
    YourGroupHostEvent : {
        event: (obj, args, context) => {
            return connectors.getEvent(context.user, obj.eventuid);
        },
        sender: (obj, args, context) => {
            return connectors.getGroup(context.user, obj.senderuid);
        }
    },

    // @rights user
    User : {
        groups : (obj, args, context) => {
            let result = Promise.all(obj.groups.map((grid) => {
                return connectors.getSimpleGroup(context.user,grid);
            }));

            return result.then(groups => {
                return _.filter(groups,(o) => !_.isUndefined(o));
            });
        },

    },

    // @rights user
    Mutation: {

        asSuperviser : function (obj, args, context){
            return {groupUID : args.groupUID, justification : "kes"};
        },

        asAdmin: (obj, args, context) => {
            return connectors.hasAdminRights(context.user, args.groupUID).then(res => {
                if(res)
                    return {groupUID : args.groupUID};
                else
                    throw "You do not have admin rights over this group";
            });
        },

        asSpeaker: (obj, args, context) => {
            return connectors.hasAdminRights(context.user, args.groupUID).then(res => {
                if(res)
                    return {groupUID : args.groupUID};
                else
                    throw "You do not have publishing rights in this group";
            });
        },

        asMember: (obj, args, context) => {
            return connectors.isMember(context.user, args.groupUID).then(res => {
                if(res)
                    return {groupUID : args.groupUID};
                else
                    throw "You are  not a member of this group";
            });
        },
        asViewer: (obj, args, context) => {
            return connectors.getGroupIfVisible(context.user, args.groupUID).then(res => {
                if(res)
                    return {groupUID : args.groupUID};
                else
                    throw "You are  not a member of this group";
            });
        }
    },

    // @rights : superviser(obj.groupUID)
    SuperviserMutation: {

        runAdminOperation : function (obj, args, context){
            return obj;
        },

        takeAdminRights : async function(obj, args, context){
            console.log(obj);
            await connectors.takeAdminRights(context.user, obj.groupUID, obj.justification);
            return connectors.getGroup(context.user, obj.groupUID);
        }
    },

    // @rights admin(obj.groupUID)
    AdminMutation: {
        isAdmin: (obj, args, context) => {
            return true;
        
        },
        createSubgroup: (obj, args, context) => {
            args.parentuid = obj.groupUID;
            return connectors.createSubgroup(context.user, args);
        },

        editGroup: (obj, args, context) => {
            return null;
        }
    },

    // @rights viewer(obj.groupUID)
    ViewerMutation: {
        requestJoin: (obj, args, context) => {
            console.log(obj);
            return null;
        }
    },

    Post: {
        authors: (obj, args, context) => {
            return knex.select().from('groups').whereIn('uid',obj.authors);
        }
    },

    Message: {

        __resolveType: function(obj){
            return obj.type;
        }

    },

    Announcement: {
        forEvent : function(obj, args, context){
            // le champ is_announcement n'existe que sur les Events
            // une ligne de la bdd events peut résoudre comme évènement ou comme annonce
            if(obj.is_announcement) 
                return obj;
            else
                return null;
        }
    },

    Event: {

        startTime : function(obj){
            return obj.start_time;
        },

        endTime : function(obj){
            return obj.end_time;
        },

        asAnnouncement : function(obj, args, context){
            // le champ is_announcement indique si il existe une annonce qui va avec l'évènement
            // une ligne de la bdd events peut résoudre comme évènement ou comme annonce
            if(obj.is_announcement) 
                return obj;
            else
                return null;
        }
    },

    // @rights viewer(obj.uid)
    Group: {
        __resolveType: (obj) => {
            
            switch(obj.type){
            case "simple":
                return "SimpleGroup";
            case "meta":
                return "MetaGroup";
            default:
                return undefined;
            }
        }
    },

    // @rights viewer(obj.uid)
    SimpleGroup: {
        members: (obj, args, context) => {
            return connectors.listerMembres(context.user,obj.uid);
        }
    },

    // @rights viewer(obj.uid)
    MetaGroup: {
        members: (obj, args, context) => {
        }
    }
};
